@@ -32,6 +32,7 @@ import (
 	"github.com/MakeNowJust/heredoc"
 	imagereference "github.com/openshift/library-go/pkg/image/reference"
 	"github.com/openshift/oc/pkg/cli/image/extract"
+	"github.com/openshift/oc/pkg/cli/image/imagesource"
 )
 
 // extractTarget describes how a file in the release image can be extracted to disk.
@@ -40,9 +41,8 @@ type extractTarget struct {
 	Command  string
 	Optional bool
 
-	TargetName string
-
-	InjectReleaseImage bool
+	InjectReleaseImage   bool
+	InjectReleaseVersion bool
 
 	ArchiveFormat string
 	AsArchive     bool
@@ -150,27 +150,30 @@ func (o *ExtractOptions) extractCommand(command string) error {
 			Command: "oc",
 			Mapping: extract.Mapping{Image: "cli-artifacts", From: "usr/share/openshift/mac/oc"},
 
-			LinkTo:        []string{"kubectl"},
-			Readme:        readmeCLIUnix,
-			ArchiveFormat: "openshift-client-mac-%s.tar.gz",
+			LinkTo:               []string{"kubectl"},
+			Readme:               readmeCLIUnix,
+			InjectReleaseVersion: true,
+			ArchiveFormat:        "openshift-client-mac-%s.tar.gz",
 		},
 		{
 			OS:      "linux",
 			Command: "oc",
 			Mapping: extract.Mapping{Image: "cli", From: "usr/bin/oc"},
 
-			LinkTo:        []string{"kubectl"},
-			Readme:        readmeCLIUnix,
-			ArchiveFormat: "openshift-client-linux-%s.tar.gz",
+			LinkTo:               []string{"kubectl"},
+			Readme:               readmeCLIUnix,
+			InjectReleaseVersion: true,
+			ArchiveFormat:        "openshift-client-linux-%s.tar.gz",
 		},
 		{
 			OS:      "windows",
 			Command: "oc",
 			Mapping: extract.Mapping{Image: "cli-artifacts", From: "usr/share/openshift/windows/oc.exe"},
 
-			Readme:        readmeCLIWindows,
-			ArchiveFormat: "openshift-client-windows-%s.zip",
-			AsZip:         true,
+			Readme:               readmeCLIWindows,
+			InjectReleaseVersion: true,
+			ArchiveFormat:        "openshift-client-windows-%s.zip",
+			AsZip:                true,
 		},
 		{
 			OS:      "darwin",
@@ -290,14 +293,15 @@ func (o *ExtractOptions) extractCommand(command string) error {
 	dir := o.Directory
 	infoOptions := NewInfoOptions(o.IOStreams)
 	infoOptions.SecurityOptions = o.SecurityOptions
+	infoOptions.FileDir = o.FileDir
 	release, err := infoOptions.LoadReleaseInfo(o.From, false)
 	if err != nil {
 		return err
 	}
 	releaseName := release.PreferredName()
 	refExact := release.ImageRef
-	refExact.Tag = ""
-	refExact.ID = release.Digest.String()
+	refExact.Ref.Tag = ""
+	refExact.Ref.ID = release.Digest.String()
 	exactReleaseImage := refExact.String()
 
 	// resolve target image references to their pull specs
@@ -319,7 +323,7 @@ func (o *ExtractOptions) extractCommand(command string) error {
 			return err
 		}
 		target.Mapping.Image = spec
-		target.Mapping.ImageRef = ref
+		target.Mapping.ImageRef = imagesource.TypedImageReference{Ref: ref, Type: imagesource.DestinationRegistry}
 		if target.AsArchive {
 			willArchive = true
 			target.Mapping.Name = fmt.Sprintf(target.ArchiveFormat, releaseName)
@@ -488,15 +492,22 @@ func (o *ExtractOptions) extractCommand(command string) error {
 		}
 
 		// copy the input to disk
+		replacements := []replacement{}
 		if target.InjectReleaseImage {
-			var matched bool
-			matched, err = copyAndReplaceReleaseImage(w, r, 4*1024, exactReleaseImage)
-			if !matched {
-				fmt.Fprintf(o.ErrOut, "warning: Unable to replace release image location into %s, installer will not be locked to the correct image\n", target.TargetName)
-			}
-		} else {
-			_, err = io.Copy(w, r)
+			replacements = append(replacements, replacement{
+				name:   "release image",
+				marker: append([]byte{0}, []byte(releaseImageMarker[1:])...),
+				value:  exactReleaseImage,
+			})
 		}
+		if target.InjectReleaseVersion {
+			replacements = append(replacements, replacement{
+				name:   "release version",
+				marker: append([]byte{0}, []byte(releaseVersionMarker[1:])...),
+				value:  releaseName,
+			})
+		}
+		err = copyAndReplace(o.ErrOut, w, r, 4*1024, replacements, target.Command)
 		if err != nil {
 			closeFn()
 			f.Close()
@@ -612,27 +623,46 @@ func (o *ExtractOptions) extractCommand(command string) error {
 }
 
 const (
-	// installerReplacement is the location within the installer binary that we can insert our
-	// release payload string
-	installerReplacement = "\x00_RELEASE_IMAGE_LOCATION_\x00XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX\x00"
+	// releaseImageMarker is the placeholder within a binary for the release image pullspec.
+	releaseImageMarker = "!_RELEASE_IMAGE_LOCATION_\x00XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX\x00"
+	// releaseVersionMarker is the placeholder within a binary for the release image version name string.
+	releaseVersionMarker = "!_RELEASE_VERSION_LOCATION_\x00XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX\x00"
 )
 
-// copyAndReplaceReleaseImage performs a targeted replacement for binaries that contain a special marker string
-// as a constant, replacing the marker with releaseImage and a NUL terminating byte. It returns true if the
-// replacement was performed.
-func copyAndReplaceReleaseImage(w io.Writer, r io.Reader, bufferSize int, releaseImage string) (bool, error) {
-	if len(releaseImage)+1 > len(installerReplacement) {
-		return false, fmt.Errorf("the release image pull spec is longer than the maximum replacement length for the installer binary")
+type replacement struct {
+	name   string
+	marker []byte
+	value  string
+}
+
+// copyAndReplace performs a targeted replacement for binaries that
+// contain special marker strings, replacing the first occurrence of each
+// marker with a new string and a NUL terminating byte.  It logs a warning
+// if any replacements are not performed.
+func copyAndReplace(errorOutput io.Writer, w io.Writer, r io.Reader, bufferSize int, replacements []replacement, name string) error {
+	if len(replacements) == 0 {
+		_, err := io.Copy(w, r)
+		return err
 	}
-	if bufferSize < len(installerReplacement) {
-		return false, fmt.Errorf("the buffer size must be greater than %d bytes", len(installerReplacement))
+
+	longestMarker := 0
+	for _, replacement := range replacements {
+		if len(replacement.value) > len(replacement.marker)-1 {
+			return fmt.Errorf("the %s value has %d bytes, but the maximum replacement length is %d", replacement.name, len(replacement.value), len(replacement.marker)-1)
+		}
+
+		if len(replacement.marker) > longestMarker {
+			longestMarker = len(replacement.marker)
+			if bufferSize < longestMarker {
+				return fmt.Errorf("the buffer size must be greater than %d bytes to find %s", len(replacement.marker), replacement.name)
+			}
+		}
 	}
 
-	match := []byte(installerReplacement[:len(releaseImage)+1])
 	offset := 0
-	max := bufferSize
-	buf := make([]byte, max+offset)
-	matched := false
+	buf := make([]byte, bufferSize)
+	remaining := len(replacements)
+	matches := make([]bool, remaining)
 
 	for {
 		n, err := io.ReadFull(r, buf[offset:])
@@ -640,42 +670,66 @@ func copyAndReplaceReleaseImage(w io.Writer, r io.Reader, bufferSize int, releas
 		// search in the buffer for the expected match
 		end := offset + n
 		if n > 0 {
-			index := bytes.Index(buf[:end], match)
-			if index != -1 {
-				klog.V(2).Infof("Found match at %d (len=%d, offset=%d, n=%d)", index, len(buf), offset, n)
-				// the replacement starts at the beginning of the match, contains the release string and a terminating NUL byte
-				copy(buf[index:index+len(releaseImage)], []byte(releaseImage))
-				buf[index+len(releaseImage)] = 0x00
-				matched = true
+			for i, matched := range matches {
+				if !matched {
+					replacement := replacements[i]
+					index := bytes.Index(buf[:end], replacement.marker)
+					if index != -1 {
+						klog.V(2).Infof("Found match for %s at %d (len=%d, offset=%d, n=%d)", replacements[i].name, index, len(buf), offset, n)
+						// the replacement starts at the beginning of the match, contains the replacement value and a terminating NUL byte
+						copy(buf[index:index+len(replacement.value)], []byte(replacement.value))
+						buf[index+len(replacement.value)] = 0x00
+						matches[i] = true
+						remaining -= 1
+					}
+				}
 			}
 		}
 
 		// write everything that we have already searched (excluding the end of the buffer that will
 		// be checked next pass)
-		nextOffset := end - len(installerReplacement)
-		if nextOffset < 0 || matched {
-			nextOffset = 0
+		writeTo := end - longestMarker
+		if writeTo < 0 {
+			writeTo = 0
 		}
-		_, wErr := w.Write(buf[:end-nextOffset])
-		if wErr != nil {
-			return matched, wErr
+		if remaining == 0 || err != nil {
+			writeTo = end
 		}
-		if err != nil {
-			if err == io.EOF || err == io.ErrUnexpectedEOF {
-				return matched, nil
+		offset = 0
+		for offset < writeTo {
+			n, wErr := w.Write(buf[offset:writeTo])
+			if wErr != nil {
+				return wErr
 			}
-			return matched, err
+			offset += n
 		}
 
-		// once we complete a single match, we can copy the rest of the file without processing
-		if matched {
+		if err != nil {
+			if err == io.EOF || err == io.ErrUnexpectedEOF {
+				remainingNames := make([]string, 0, len(replacements))
+				for i, matched := range matches {
+					if !matched {
+						remainingNames = append(remainingNames, replacements[i].name)
+					}
+				}
+				sort.Strings(remainingNames)
+				if len(remainingNames) > 0 {
+					fmt.Fprintf(errorOutput, "warning: Unable to make all expected replacements in %s.  Remaining: %s", name, strings.Join(remainingNames, ", "))
+				}
+				return nil
+			}
+			return err
+		}
+
+		// once we match each replacement, we can copy the rest of the file without processing
+		if remaining == 0 {
 			_, err := io.Copy(w, r)
-			return matched, err
+			return err
 		}
 
 		// ensure the beginning of the buffer matches the end of the current buffer so that we
 		// can search for matches that span buffers
-		copy(buf[:nextOffset], buf[end-nextOffset:end])
-		offset = nextOffset
+		copy(buf[:writeTo], buf[writeTo:end])
+		offset = end - writeTo
 	}
 }
